#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from Products.CMFCore import permissions
try:
    from Products.LinguaPlone.public import BaseSchema, Schema
    from Products.LinguaPlone.public import MetadataSchema
    from Products.LinguaPlone.public import StringField, TextField, ReferenceField, BooleanField, DateTimeField, ImageField, LinesField, ComputedField
    from Products.LinguaPlone.public import MultiSelectionWidget, TextAreaWidget, StringWidget, RichWidget, CalendarWidget, BooleanWidget, ImageWidget, LinesWidget, ReferenceWidget, InAndOutWidget, ComputedWidget
    from Products.LinguaPlone.public import AnnotationStorage
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import BaseSchema, Schema
    from Products.Archetypes.public import MetadataSchema
    from Products.Archetypes.public import StringField, TextField, ReferenceField, BooleanField, DateTimeField, ImageField, LinesField, ComputedField
    from Products.Archetypes.public import MultiSelectionWidget, TextAreaWidget, StringWidget, RichWidget, CalendarWidget, BooleanWidget, ImageWidget, LinesWidget, ReferenceWidget, InAndOutWidget, ComputedWidget
    from Products.Archetypes.public import AnnotationStorage

from DateTime import DateTime
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
from Products.ATContentTypes.atct import ATTopicSchema

relatedItemsField = ReferenceField('relatedItems',
        relationship = 'relatesTo',
        multiValued = True,
        isMetadata = True,
        languageIndependent = False,
        index = 'KeywordIndex',
        write_permission = permissions.ModifyPortalContent,
        widget = ReferenceBrowserWidget(
            allow_search = True,
	    allow_browse = True,
	    show_indexes = False,
	    force_close_on_insert = False,
	    label = "Related Item(s)",
	    label_msgid = "label_related_items",
	    description = "Reference other items on your site here.",
	    description_msgid = "help_related_items",
	    i18n_domain = "plone",
	    visible = {'edit' : 'visible', 'view' : 'invisible' }
	)
)
	
ATResearchProjectSchema =  BaseSchema.copy() +  Schema((
        StringField(name='title',
		required=1,
		searchable=1,
		default='',
		mutator='setResearchProjectOfficialTitleAndTitle',
		accessor='Title',
		widget=StringWidget(
		    label="Title",
		    label_msgid='label_title',
		    description = "",
		    description_msgid = "help_title_researchproject",
		    visible={'view': 'invisible'},
		    i18n_domain = 'plone',
		),
	),	
	TextField('researchProjectOfficialTitle',
	        required=True,
                searchable=1,
		mutator='setResearchProjectOfficialTitleAndDescription',
                widget=TextAreaWidget(
		    label='Official Title of this Research Project',
		    label_msgid = "label_officialtitle_researchproject",
  		    description="",
		    description_msgid = "help_officialtitle_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('authors',
                searchable=0,
		languageIndependent = False, 
                validators=('haveValidCharacters','haveValidExtendedFieldstructureTags',),
                widget=LinesWidget(
		    label='Authors',
		    label_msgid = "label_authors_researchproject",
  		    description="",
		    description_msgid = "help_authors_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	StringField('researchProjectAbstract',
		required=False,
                searchable=1,
                widget=TextAreaWidget(
		    label='Abstract',
		    label_msgid = "label_abstract_researchproject",
  		    description="",
		    description_msgid = "help_abstract_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchProjectKeywords',
		required=False,
                searchable=1,
                widget=LinesWidget(
		    label='Keywords',
		    label_msgid = "label_keywords_researchproject",
  		    description="",
		    description_msgid = "help_keywords_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	DateTimeField('researchProjectRuntimeStart',
	        required=True,
		default_method=DateTime,
		languageIndependent = True, 
                widget=CalendarWidget(
		    label="Begin of Project Runtime",
		    label_msgid="label_runtimestart_researchproject",
		    description="",
		    description_msgid = "help_calendar_researchproject",
		    i18n_domain = 'atresearchproject',
  	  	    show_hm=False,
		    starting_year=1980,
		    future_years=20,
		),
        ),
	DateTimeField('researchProjectRuntimeEnd',
	        required=False,
		default_method=None,
		languageIndependent = True, 
                widget=CalendarWidget(
		    label='End of Project Runtime',
		    label_msgid = "label_runtimeend_researchproject",
		    description='',
		    description_msgid = "help_calendar_researchproject",
		    i18n_domain = 'atresearchproject',
		    show_hm=False,
		    starting_year=1980,
		    future_years=20,
		),
        ),
#        ReferenceField('researchProjectResearchFields',
#		relationship = 'research fields',
#                isMetadata = False,
#                languageIndependent = False,
#                multiValued = True,
#		allowed_types = ('ResearchField',),
#		vocabulary_custom_label = "b.getObject().getResearchFieldRefFieldEntry()",
#		write_permission = permissions.ModifyPortalContent,
#		widget=ReferenceWidget(
#		    label = 'Research Field Context',
#		    label_msgid = "label_researchfields_researchproject",
#          	    description = "",
#		    description_msgid = "help_researchfields_researchproject",
#                    macro = "research_project_researchfieldreference_widget",
#                    checkbox_bound = 0,
#		    i18n_domain = 'atresearchproject',
#		)
#	),
        ReferenceField('researchProjectResearchFields',
		relationship = 'research fields',
                isMetadata = False,
                languageIndependent = False,
                multiValued = True,
		allowed_types = ('ResearchField',),
		vocabulary_custom_label = "b.getObject().getResearchFieldRefFieldEntry()",
		write_permission = permissions.ModifyPortalContent,
		widget=InAndOutWidget(
		    label = 'Research Field Context',
		    label_msgid = "label_researchfields_researchproject",
          	    description = "",
		    description_msgid = "help_researchfields_researchproject",
                    macro = "research_project_researchfieldreference_inandoutwidget",
                    helper_js = ('research_project_inandout.js',),
                    checkbox_bound = 0,
		    i18n_domain = 'atresearchproject',
		    condition = 'python: here.portal_researchprojects.siteHasResearchFields()',
		)
	),
	LinesField('researchProjectInvolvedDepartments',
                searchable=0,
		languageIndependent = True, 
                widget=InAndOutWidget(
		    size=6,
		    label='Involved Departments',
		    label_msgid = "label_involveddepartments_researchproject",
  		    description="",
		    description_msgid = "help_involveddepartments_researchproject",
                    macro = "involveddepartments_inandoutwidget",
                    helper_js = ('involveddepartments_inandout.js',),
		    i18n_domain = 'atresearchproject',
		),
        ),
	# generic field (hidden)
	ComputedField('researchProjectInfoFields',
                searchable=0,
		languageIndependent = False, 
                widget=ComputedWidget(
		    i18n_domain = 'atresearchproject'
		),
        ),
	LinesField('researchProjectContactPersons',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='Contact',
		    label_msgid = "label_contactpersons_researchproject",
  		    description="",
		    description_msgid = "help_contactpersons_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchProjectProjectCoordinators',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='Project Coordination',
		    label_msgid = "label_projectcoordinators_researchproject",
  		    description="",
		    description_msgid = "help_projectcoordinators_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchProjectScientificCoordinators',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='Scientific Coordination',
		    label_msgid = "label_scientificcoordinators_researchproject",
  		    description="",
		    description_msgid = "help_scientificcoordinators_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchProjectScientificStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchProjectInheritedScientificStaffMembers',
                edit_accessor = 'editResearchProjectScientificStaffMembers',
                widget=LinesWidget(
		    label='Scientific Staff Members',
		    label_msgid = "label_scientificstaffmembers_researchproject",
  		    description="",
		    description_msgid = "help_scientificstaffmembers_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchProjectInheritScientificStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Scientific Staff Member Lists",
                    label_msgid="label_inherit_scientificstaffmembers_researchproject",
                    description="",
                    description_msgid="help_inherit_scientificstaffmembers_researchproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchProjectTechnicalStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchProjectInheritedTechnicalStaffMembers',
                edit_accessor = 'editResearchProjectTechnicalStaffMembers',
		widget=LinesWidget(
		    label='Technical Staff Members',
		    label_msgid = "label_technicalstaffmembers_researchproject",
  		    description="",
		    description_msgid = "help_technicalstaffmembers_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchProjectInheritTechnicalStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Technical Staff Member Lists",
                    label_msgid="label_inherit_technicalstaffmembers_researchproject",
                    description="",
                    description_msgid="help_inherit_technicalstaffmembers_researchproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchProjectStudentStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchProjectInheritedStudentStaffMembers',
                edit_accessor = 'editResearchProjectStudentStaffMembers',
                widget=LinesWidget(
		    label='Student Staff Members',
		    label_msgid = "label_studentstaffmembers_researchproject",
  		    description="",
		    description_msgid = "help_studentstaffmembers_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchProjectInheritStudentStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Student Staff Member Lists",
                    label_msgid="label_inherit_studentstaffmembers_researchproject",
                    description="",
                    description_msgid="help_inherit_studentstaffmembers_researchproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchProjectFormerStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchProjectInheritedFormerStaffMembers',
                edit_accessor = 'editResearchProjectFormerStaffMembers',
                widget=LinesWidget(
		    label='Former Staff Members',
		    label_msgid = "label_formerstaffmembers_researchproject",
  		    description="",
		    description_msgid = "help_formerstaffmembers_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchProjectInheritFormerStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Former Staff Member Lists",
                    label_msgid="label_inherit_formerstaffmembers_researchproject",
                    description="",
                    description_msgid="help_inherit_formerstaffmembers_researchproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchProjectFormerStudentStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchProjectInheritedFormerStudentStaffMembers',
                edit_accessor = 'editResearchProjectFormerStudentStaffMembers',
                widget=LinesWidget(
		    label='Former Student Staff Members',
		    label_msgid = "label_formerstudentstaffmembers_researchproject",
  		    description="",
		    description_msgid = "help_formerstudentstaffmembers_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchProjectInheritFormerStudentStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Former Student Staff Member Lists",
                    label_msgid="label_inherit_formerstudentstaffmembers_researchproject",
                    description="",
                    description_msgid="help_inherit_formerstudentstaffmembers_researchproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchProjectCooperationPartners',
                searchable=0,
		languageIndependent = False, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='In Cooperation with',
		    label_msgid = "label_cooperationpartners_researchproject",
  		    description="",
		    description_msgid = "help_cooperationpartners_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchProjectContractors',
                searchable=0,
		languageIndependent = False, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='Contractor(s) or contracting Organization(s)',
		    label_msgid = "label_contractors_researchproject",
  		    description="",
		    description_msgid = "help_contractors_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	BooleanField('researchProjectThirdPartyFunded',
                searchable=0,
		languageIndependent = True, 
		default=True,
                widget=BooleanWidget(
		    label='Third-party funded research project',
		    label_msgid = "label_thirdpartyfunded_researchproject",
  		    description="",
		    description_msgid = "help_thirdpartyfunded_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchProjectExternalLinks',
                searchable=0,
		languageIndependent = False, 
		validators=('areURLsOrHaveURLTags',),
                widget=LinesWidget(
		    label='External internet links with further information on this research project',
		    label_msgid = "label_externallinks_researchproject",
  		    description="",
		    description_msgid = "help_externallinks_researchproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	TextField('body',
                searchable=1,
                required=1,
                primary=1,
                default_content_type='text/html',
                default_output_type='text/x-html-captioned',
                allowable_content_types=('text/html',
                                         'text/plain',
                                         ),
                widget=RichWidget(
                    label='More specific Information on this Research Project',
		    label_msgid = "label_body_researchproject",
  		    description="",
		    description_msgid = "help_body_researchproject",
  	            rows=15,
		    i18n_domain = 'atresearchproject',
		),		  
        ),
	ImageField('logo',
	       display_threshold = 800,
	       storage = AnnotationStorage(migrate=True),
               sizes = {'large'   : (768, 768),
    		        'preview' : (400, 400),
		        'mini'    : (200, 200),
		        'thumb'   : (128, 128),
		        'tile'    :  (64, 64),
		        'icon'    :  (32, 32),
		        'listing' :  (16, 16),
		},
		languageIndependent = True, 
	        widget=ImageWidget(
		    label='Logo (optional)',
		    label_msgid="label_logo_researchproject",
		    description="",
		    description_msgid="help_logo_researchproject",
		    i18n_domain = 'atresearchproject',
		    showContentType = True, 
		),
	),        
	relatedItemsField,
      ),
      marshall=PrimaryFieldMarshaller(),
    ) + MetadataSchema((
	  BooleanField('excludeFromNav',
	    required = False,
	    languageIndependent = True,
	    schemata = 'metadata', # moved to 'default' for folders
	    widget = BooleanWidget(
		description="",
		description_msgid = "help_exclude_from_nav",
	    	label = "Exclude from navigation",
	        label_msgid = "label_exclude_from_nav",
	        i18n_domain = "plone",
		visible={'view' : 'hidden',
	    	         'edit' : 'visible'},
	    ),	     
	  ),
        )
    )  
     
     
ATResearchSubprojectSchema =  BaseSchema.copy() +  Schema((
        StringField(name='title',
		required=1,
		searchable=1,
		default='',
		mutator='setResearchSubprojectOfficialTitleAndTitle',
		accessor='Title',
		widget=StringWidget(
                    label = "",
		    label_msgid='label_title',
		    description="",
		    description_msgid = "help_title_researchsubproject",
		    visible={'view': 'invisible'},
		    i18n_domain = 'plone',
		),
	),	
	TextField('description',
		required=1,
                searchable=1,
		mutator='setResearchSubprojectOfficialTitleAndDescription',
                widget=TextAreaWidget(
		    label='Official Title of this Subproject',
		    label_msgid = "label_officialtitle_researchsubproject",
  		    description="",
		    description_msgid = "help_officialtitle_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('authors',
                searchable=0,
		languageIndependent = False, 
                validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='Authors',
		    label_msgid = "label_authors_researchsubproject",
  		    description="",
		    description_msgid = "help_authors_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	StringField('researchSubprojectAbstract',
                required=False,
                searchable=1,
                widget=TextAreaWidget(
		    label='Abstract',
		    label_msgid = "label_abstract_researchsubproject",
  		    description="",
		    description_msgid = "help_abstract_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchSubprojectKeywords',
		required=False,
                searchable=1,
                widget=LinesWidget(
		    label='Keywords',
		    label_msgid = "label_keywords_researchsubproject",
  		    description="",
		    description_msgid = "help_keywords_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	DateTimeField('researchSubprojectRuntimeStart',
	        required=False,
		default_method='getResearchProjectRuntimeStart',
		languageIndependent = True, 
                widget=CalendarWidget(
		    label='Begin of Sub-Project Runtime',
		    label_msgid = "label_runtimestart_researchproject",
		    description='',
		    description_msgid = "help_calendar_researchproject",
		    i18n_domain = 'atresearchproject',
		    show_hm=False,
		    starting_year=1980,
		    future_years=20,
		),
        ),
	DateTimeField('researchSubprojectRuntimeEnd',
	        required=False,
		default_method='getResearchProjectRuntimeEnd',
		languageIndependent = True, 
                widget=CalendarWidget(
		    label='End of Sub-Project Runtime',
		    label_msgid = "label_runtimeend_researchproject",
		    description='',
		    description_msgid = "help_calendar_researchproject",
		    i18n_domain = 'atresearchproject',
		    show_hm=False,
		    starting_year=1980,
		    future_years=20,
		),
        ),
	LinesField('researchSubprojectInvolvedDepartments',
                searchable=0,
		languageIndependent = True, 
                widget=InAndOutWidget(
		    size=6,
		    label='Involved Departments',
		    label_msgid = "label_involveddepartments_researchsubproject",
  		    description="",
		    description_msgid = "help_involveddepartments_researchsubproject",
                    macro = "involveddepartments_inandoutwidget",
                    helper_js = ('involveddepartments_inandout.js',),
		    i18n_domain = 'atresearchproject',
		),
        ),
	# generic field (hidden)
	ComputedField('researchSubprojectInfoFields',
                searchable=0,
		languageIndependent = False, 
                widget=ComputedWidget(
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchSubprojectScientificCoordinators',
                searchable=0,
		languageIndependent = True, 
                validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='Scientific Coordination',
		    label_msgid = "label_scientificcoordinators_researchsubproject",
  		    description="",
		    description_msgid = "help_scientificcoordinators_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchSubprojectScientificStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchSubprojectInheritedScientificStaffMembers',
		edit_accessor = 'editResearchSubprojectScientificStaffMembers',
                widget=LinesWidget(
		    label='Scientific Staff Members',
		    label_msgid = "label_scientificstaffmembers_researchsubproject",
  		    description="",
		    description_msgid = "help_scientificstaffmembers_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchSubprojectInheritScientificStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Scientific Staff Member Lists",
                    label_msgid="label_inherit_scientificstaffmembers_researchsubproject",
                    description="",
                    description_msgid="help_inherit_scientificstaffmembers_researchsubproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchSubprojectTechnicalStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchSubprojectInheritedTechnicalStaffMembers',
		edit_accessor = 'editResearchSubprojectTechnicalStaffMembers',
                widget=LinesWidget(
		    label='Technical Staff Members',
		    label_msgid = "label_technicalstaffmembers_researchsubproject",
  		    description="",
		    description_msgid = "help_technicalstaffmembers_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchSubprojectInheritTechnicalStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Technical Staff Member Lists",
                    label_msgid="label_inherit_technicalstaffmembers_researchsubproject",
                    description="",
                    description_msgid="help_inherit_technicalstaffmembers_researchsubproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchSubprojectStudentStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators = ('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchSubprojectInheritedStudentStaffMembers',
		edit_accessor = 'editResearchSubprojectStudentStaffMembers',
                widget=LinesWidget(
		    label='Student Staff Members',
		    label_msgid = "label_studentstaffmembers_researchsubproject",
  		    description="",
		    description_msgid = "help_studentstaffmembers_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchSubprojectInheritStudentStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Student Staff Member Lists",
                    label_msgid="label_inherit_studentstaffmembers_researchsubproject",
                    description="",
                    description_msgid="help_inherit_studentstaffmembers_researchsubproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchSubprojectFormerStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchSubprojectInheritedFormerStaffMembers',
		edit_accessor = 'editResearchSubprojectFormerStaffMembers',
                widget=LinesWidget(
		    label='Former Staff Members',
		    label_msgid = "label_formerstaffmembers_researchsubproject",
  		    description="",
		    description_msgid = "help_formerstaffmembers_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchSubprojectInheritFormerStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Former Staff Member Lists",
                    label_msgid="label_inherit_formerstaffmembers_researchsubproject",
                    description="",
                    description_msgid="help_inherit_formerstaffmembers_researchsubproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchSubprojectFormerStudentStaffMembers',
                searchable=0,
		languageIndependent = True, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
		accessor = 'getResearchSubprojectInheritedFormerStudentStaffMembers',
		edit_accessor = 'editResearchSubprojectFormerStudentStaffMembers',
                widget=LinesWidget(
		    label='Former Student Staff Members',
		    label_msgid = "label_formerstudentstaffmembers_researchsubproject",
  		    description="",
		    description_msgid = "help_formerstudentstaffmembers_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchSubprojectInheritFormerStudentStaffMembers',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Former Student Staff Member Lists",
                    label_msgid="label_inherit_formerstudentstaffmembers_researchsubproject",
                    description="",
                    description_msgid="help_inherit_formerstudentstaffmembers_researchsubproject",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchSubprojectExternalInstitutes',
                searchable=0,
		languageIndependent = False, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='External Institute(s)',
		    label_msgid = "label_externalinstitutes_researchsubproject",
  		    description="",
		    description_msgid = "help_externalinstitutes_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchSubprojectCooperationPartners',
                searchable=0,
		languageIndependent = False, 
		validators=('haveValidCharacters','haveValidExtendedFieldstructureTags', ),
                widget=LinesWidget(
		    label='In Cooperation with',
		    label_msgid='label_cooperationpartners_researchsubproject',
  		    description="",
		    description_msgid = "help_cooperationpartners_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	LinesField('researchSubprojectExternalLinks',
                searchable=0,
		languageIndependent = False, 
		validators=('areURLsOrHaveURLTags',),
                widget=LinesWidget(
		    label='External internet links with further information on this subproject',
    		    label_msgid = "label_externallinks_researchsubproject",
  		    description="",
    		    description_msgid = "help_externallinks_researchsubproject",
		    i18n_domain = 'atresearchproject',
		),
        ),
	TextField('body',
                searchable=1,
                required=1,
                primary=1,
                default_content_type='text/html',
                default_output_type='text/x-html-captioned',
                allowable_content_types=('text/html',
                                         'text/plain',),
                widget=RichWidget(
                    label='More specific Information on this Research Subproject',
		    label_msgid = "label_body_researchsubproject",
  		    description="",
		    description_msgid = "help_body_researchsubproject",
                    rows=25,
		    i18n_domain = 'atresearchproject',
		),		  
        ),
        relatedItemsField,
      ),
      marshall=PrimaryFieldMarshaller(),
    ) + MetadataSchema((
	  BooleanField('excludeFromNav',
	    required = False,
	    languageIndependent = True,
	    schemata = 'metadata', # moved to 'default' for folders
	    widget = BooleanWidget(
	    	label = "Exclude from navigation",
	        label_msgid = "label_exclude_from_nav",
		description="If selected, this item will not appear in the navigation tree",
		description_msgid = "help_exclude_from_nav",
	        i18n_domain = "plone",
		visible={'view' : 'hidden',
	    	         'edit' : 'visible'},
	    ),	     
	  ),
        )
    )  

ATResearchProjectListSchema =  ATTopicSchema.copy() +  Schema((
        StringField(name='title',
		required=1,
		searchable=1,
		default='',
		accessor='Title',
		widget=StringWidget(
                    label = "",
		    label_msgid='label_title',
		    description="",
		    description_msgid = "help_title_researchprojectlist",
		    visible={'view': 'invisible'},
		    i18n_domain = 'plone',
		),
	),	
	TextField('description',
                searchable = True,
		required = False,
                accessor = 'Description',
                widget=TextAreaWidget(
		    label='Description',
		    label_msgid='label_description_researchprojectlist',
		    description="",
                    description_msgid="help_description_researchprojectlist",
		    i18n_domain = 'atresearchproject',
		),
        ),
	StringField('researchProjectListComment',
                searchable=1,
		languageIndependent = False, 
                widget=TextAreaWidget(
		    label='Comment',
		    label_msgid='label_comment_researchprojectlist',
  		    description="",
		    description_msgid = "help_comment_researchprojectlist",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('acquireCriteria',
                required=False,
                mode="rw",
                default=True,
		languageIndependent=True,
                widget=BooleanWidget(
                    label="Inherit Criteria",
                    label_msgid="label_inherit_criteria",
                    description=("Narrow down the search results from the parent Smart Folder(s) "
                                 "by using the criteria from this Smart Folder."),
                    description_msgid="help_inherit_criteria",
                    i18n_domain = "plone",
                    condition = "python:folder.getParentNode().portal_type == 'ResearchProjectList'",
                ),
        ),        
        BooleanField('researchProjectListDisplayResearchProjects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Research Projects",
                    label_msgid="label_display_researchprojects_researchprojectlist",
                    description="",
                    description_msgid="help_display_researchprojects_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayHeadlineResearchProjects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Headline above Research Project Listing",
                    label_msgid="label_display_rpheadline_researchprojectlist",
                    description="",
                    description_msgid="help_display_rpheadline_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayRuntimeResearchProjects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Project Runtime",
                    label_msgid="label_display_rpruntime_researchprojectlist",
                    description="",
                    description_msgid="help_display_rpruntime_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayProjectCoordinatorsResearchProjects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Project Coordinators of Research Projects",
                    label_msgid="label_display_rpprojectcoordinators_researchprojectlist",
                    description="",
                    description_msgid="help_display_rpprojectcoordinators_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayContactPersonsResearchProjects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Contact Persons of Research Projects",
                    label_msgid="label_display_rpcontactpersons_researchprojectlist",
                    description="",
                    description_msgid="help_display_rpcontactpersons_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayContractorsResearchProjects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Project Contractors of Research Projects",
                    label_msgid="label_display_rpcontractors_researchprojectlist",
                    description="",
                    description_msgid="help_display_rpcontractors_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayAbstractsResearchProjects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Abstracts of Research Projects",
                    label_msgid="label_display_rpabstracts_researchprojectlist",
                    description="",
                    description_msgid="help_display_rpabstracts_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayResearchSubprojects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Research Subprojects",
                    label_msgid="label_display_researchsubprojects_researchprojectlist",
                    description="",
                    description_msgid="help_display_researchsubprojects_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayHeadlineResearchSubprojects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Headline above Research Subproject Listing",
                    label_msgid="label_display_rspheadline_researchprojectlist",
                    description="",
                    description_msgid="help_display_rspheadline_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayRuntimeResearchSubprojects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Subproject Runtime",
                    label_msgid="label_display_rspruntime_researchprojectlist",
                    description="",
                    description_msgid="help_display_rspruntime_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayScientificCoordinatorsResearchSubprojects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Scientific Coordinators of Subprojects",
                    label_msgid="label_display_rspscientificcoordinators_researchprojectlist",
                    description="",
                    description_msgid="help_display_rspscientificcoordinators_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListDisplayAbstractsResearchSubprojects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Abstracts of Research Subprojects",
                    label_msgid="label_display_rspabstracts_researchprojectlist",
                    description="",
                    description_msgid="help_display_rspabstracts_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListCascadeResearchSubprojects',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Cascade Research Subprojects",
                    label_msgid="label_cascade_researchsubprojects_researchprojectlist",
                    description="",
                    description_msgid="help_cascade_researchsubprojects_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchProjectListViewNestedLists',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="View nested Research Project Lists",
                    label_msgid="label_view_nested_lists_researchprojectlist",
                    description="",
                    description_msgid="help_view_nested_lists_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
	StringField('researchProjectListCustomNestedListsHead',
		default='',
		required=False,
                searchable=0,
                widget=StringWidget(
		    label='Customize Headline for nested Research Project Lists',
                    label_msgid="label_custom_nested_lists_head_researchprojectlist",
                    description="",
                    description_msgid="help_custom_nested_lists_head_researchprojectlist",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchProjectListFilterWorkflowStates',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Filter Workflow States",
                    label_msgid="label_filterworkflowstates_researchprojectlist",
                    description="",
                    description_msgid="help_filterworkflowstates_researchprojectlist",
        	    i18n_domain = 'atresearchproject',
		),
	),
        relatedItemsField,
    ))
ATResearchProjectListSchema['limitNumber'].languageIndependent = True
ATResearchProjectListSchema['itemCount'].languageIndependent = True
ATResearchProjectListSchema['customView'].languageIndependent = True
ATResearchProjectListSchema['customViewFields'].languageIndependent = True
ATResearchProjectListSchema.moveField('acquireCriteria', after='description')
ATResearchProjectListSchema.moveField('researchProjectListComment', after='acquireCriteria')
ATResearchProjectListSchema.moveField('researchProjectListDisplayResearchProjects', after='researchProjectListComment')
ATResearchProjectListSchema.moveField('researchProjectListDisplayHeadlineResearchProjects', after='researchProjectListDisplayResearchProjects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayRuntimeResearchProjects', after='researchProjectListDisplayHeadlineResearchProjects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayProjectCoordinatorsResearchProjects', after='researchProjectListDisplayRuntimeResearchProjects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayContactPersonsResearchProjects', after='researchProjectListDisplayProjectCoordinatorsResearchProjects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayContractorsResearchProjects', after='researchProjectListDisplayContactPersonsResearchProjects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayAbstractsResearchProjects', after='researchProjectListDisplayContractorsResearchProjects')

ATResearchProjectListSchema.moveField('researchProjectListDisplayResearchSubprojects', after='researchProjectListDisplayAbstractsResearchProjects')
ATResearchProjectListSchema.moveField('researchProjectListCascadeResearchSubprojects', after='researchProjectListDisplayResearchSubprojects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayHeadlineResearchSubprojects', after='researchProjectListCascadeResearchSubprojects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayRuntimeResearchSubprojects', after='researchProjectListDisplayHeadlineResearchSubprojects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayScientificCoordinatorsResearchSubprojects', after='researchProjectListDisplayRuntimeResearchSubprojects')
ATResearchProjectListSchema.moveField('researchProjectListDisplayAbstractsResearchSubprojects', after='researchProjectListDisplayScientificCoordinatorsResearchSubprojects')

ATResearchFieldSchema =  BaseSchema.copy() +  Schema((
        StringField(name='title',
		required = True,
		searchable = True,
		default='',
		accessor='Title',
		widget=StringWidget(
		    label="",
		    label_msgid='label_title',
		    description="",
		    description_msgid = "help_title_researchfield",
		    visible={'view': 'invisible'},
		    i18n_domain = 'plone',
		),
	),	
	TextField('description',
                searchable = True,
		required = False,
                accessor = 'Description',
                widget=TextAreaWidget(
		    label='Description',
		    label_msgid='label_description_researchfield',
		    description="",
                    description_msgid="help_description_researchfield",
		    i18n_domain = 'atresearchproject',
		),
        ),
        BooleanField('researchFieldDisplayProjectRuntime',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Research Project Runtime",
                    label_msgid="label_display_projectruntime_researchfield",
                    description="",
                    description_msgid="help_display_projectruntime_researchfield",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchFieldDisplayProjectCoordinators',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Project Coordinators of Research Projects",
                    label_msgid="label_display_projectcoordinators_researchfield",
                    description="",
                    description_msgid="help_display_projectcoordinators_researchfield",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchFieldDisplayContactPersons',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Contact Person(s) of Research Projects",
                    label_msgid="label_display_contactpersons_researchfield",
                    description="",
                    description_msgid="help_display_contactpersons_researchfield",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchFieldDisplayContractors',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Project Contractors",
                    label_msgid="label_display_contractors_researchfield",
                    description="",
                    description_msgid="help_display_contractors_researchfield",
        	    i18n_domain = 'atresearchproject',
		),
	),
        BooleanField('researchFieldDisplayAbstract',
                required=False,
                mode="rw",
                default=True,
                languageIndependent=True,
                widget=BooleanWidget(
                    label="Display Abstracts of Research Projects",
                    label_msgid="label_display_abstracts_researchfield",
                    description="",
                    description_msgid="help_display_abstracts_researchfield",
        	    i18n_domain = 'atresearchproject',
		),
	),
	LinesField('researchFieldExternalLinks',
                searchable=0,
		languageIndependent = False, 
		validators=("areURLsOrHaveURLTags",),
                widget=LinesWidget(
		    label='External internet links with further information on this research field',
		    label_msgid='label_externallinks_researchfield',
  		    description="",
		    description_msgid = "help_externallinks_researchfield",
		    i18n_domain = 'atresearchproject',
		),
        ),
	TextField('body',
                searchable=1,
                required=1,
                primary=1,
                default_content_type='text/html',
                default_output_type='text/x-html-captioned',
                allowable_content_types=('text/html',
                                         'text/plain',
                                         ),
                widget=RichWidget(label='Body Text',
		                  label_msgid="label_body_text",
  	                          rows=25,
				  i18n_domain = 'plone',
		),		  
        ),
        relatedItemsField,
      ),
      marshall=PrimaryFieldMarshaller(),
    ) + MetadataSchema((
	  BooleanField('excludeFromNav',
	    required = False,
	    languageIndependent = True,
	    schemata = 'metadata', # moved to 'default' for folders
	    widget = BooleanWidget(
		description="If selected, this item will not appear in the navigation tree",
		description_msgid = "help_exclude_from_nav",
	    	label = "Exclude from navigation",
	        label_msgid = "label_exclude_from_nav",
	        i18n_domain = "plone",
		visible={'view' : 'hidden',
	    	         'edit' : 'visible'},
	    ),	     
	  ),
        )
    )  

